<?php

namespace Drupal\layout_builder_kit\Plugin\Block\LBKIconText;

use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\file\Entity\File;
use Drupal\layout_builder_kit\Plugin\Block\LBKBaseComponent;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'LBKIconText' block.
 *
 * @Block(
 *  id = "lbk_icon_text",
 *  admin_label = @Translation("Icon Text (LBK)"),
 * )
 */
class LBKIconText extends LBKBaseComponent implements ContainerFactoryPluginInterface {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Config\ConfigManagerInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigManagerInterface
   */
  protected $configManager;

  /**
   * Constructs a new LBKIconText object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param string $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The EntityTypeManagerInterface service.
   * @param \Drupal\Core\Config\ConfigManagerInterface $config_manager
   *   The ConfigManagerInterface service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    ConfigManagerInterface $config_manager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->configManager = $config_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('config.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'grid_fields' => [
        'grid_title' => 'Icon Text (LBK)',
        'grid_display_title' => TRUE,
        'grid_image' => [],
        'grid_position_media' => 'media_on_left',
        'grid_text' => [],
        'grid_link' => '',
        'grid_classes' => '',
      ],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   *
   * Creates a generic configuration form for all block types. Individual
   * block plugins can add elements to this form by overriding
   * BlockBase::blockForm(). Most block plugins should not override this
   * method unless they need to alter the generic form elements.
   *
   * @see \Drupal\Core\Block\BlockBase::blockForm()
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $definition = $this->getPluginDefinition();
    $form['provider'] = [
        '#type' => 'value',
        '#value' => $definition['provider'],
    ];

    /*
    $form['admin_label'] = [
        '#type' => 'item',
        '#title' => $this->t('Block description'),
        '#plain_text' => $definition['admin_label'],
    ];
    */

    // Add context mapping UI form elements.
    $contexts = $form_state->getTemporaryValue('gathered_contexts') ?: [];
    $form['context_mapping'] = $this->addContextAssignmentElement($this, $contexts);
    // Add plugin-specific settings for this block type.
    $form += $this->blockForm($form, $form_state);
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    // Get value from ImageComponentSettingsForm's configuration.
    $configFactory = $this->configManager->getConfigFactory();
    $locationFolderConfig = $configFactory->get('layout_builder_kit.image_component');
    $locationFolder = $locationFolderConfig->get('image_location');

    $form['grid_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Title'),
      '#maxlength' => 255,
      '#default_value' => $this->configuration['grid_fields']['grid_title'],
      '#required' => TRUE,
      '#weight' => '1',
    ];

    $form['grid_display_title'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display title'),
      '#default_value' => $this->configuration['grid_fields']['grid_display_title'],
      '#weight' => '2',
    ];

    $form['grid_image'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Image'),
      '#upload_location' => ($locationFolder != NULL) ? 'public://' . $locationFolder : 'public://layout_builder_kit',
      '#multiple' => FALSE,
      '#default_value' => $this->configuration['grid_fields']['grid_image'],
      '#upload_validators' => [
        'file_validate_extensions' => ['gif png jpg jpeg'],
      ],
      '#weight' => '3',
    ];

    $form['grid_position_media'] = [
      '#type' => 'select',
      '#title' => $this->t('Position media'),
      '#options' => [
        'media_on_left' => $this->t('Media on Left'),
        'media_on_right' => $this->t('Media on Right'),
      ],
      '#default_value' => $this->configuration['grid_fields']['grid_position_media'],
      '#weight' => '4',
    ];

    $form['grid_link'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Link URL'),
      '#description' => $this->t('URL for title and icon.'),
      '#maxlength' => 64,
      '#default_value' => $this->configuration['grid_fields']['grid_link'],
      '#weight' => '5',
    ];

    $form['grid_text'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Text'),
      '#default_value' => $this->configuration['grid_fields']['grid_text']['value'],
      '#format' => $this->configuration['grid_fields']['grid_text']['format'],
      '#weight' => '6',
    ];

    $form['grid_classes'] = [
      '#type' => 'textfield',
      '#title' => $this->t('CSS class'),
      '#description' => $this->t('A list of CSS classes added to this component separated by spaces.'),
      '#maxlength' => 64,
      '#default_value' => $this->configuration['grid_fields']['grid_classes'],
      '#weight' => '7',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['grid_fields']['grid_title'] = $form_state->getValue('grid_title');
    $this->configuration['grid_fields']['grid_display_title'] = $form_state->getValue('grid_display_title');
    $this->configuration['grid_fields']['grid_text'] = $form_state->getValue('grid_text');
    $this->configuration['grid_fields']['grid_position_media'] = $form_state->getValue('grid_position_media');
    $this->configuration['grid_fields']['grid_image'] = $form_state->getValue('grid_image');
    $this->configuration['grid_fields']['grid_link'] = $form_state->getValue('grid_link');
    $this->configuration['grid_fields']['grid_classes'] = $form_state->getValue('grid_classes');
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    // Add template.
    $build['#theme'] = 'LBKIconText';
    // Attach library.
    $build['#attached']['library'] = ['layout_builder_kit/image-grid-styling'];

    if (!empty($this->configuration['grid_fields']['grid_image'])) {
      $gridImageFileId = implode($this->configuration['grid_fields']['grid_image']);
      $gridImage = File::load($gridImageFileId);
      if ($gridImage != NULL) {
        $gridImage->setPermanent();
        $gridImage->save();

        $build['#image'] = [
          '#theme' => 'image',
          '#uri' => $gridImage->getFileUri(),
        ];
      }
    }

    $build['#title'] = $this->configuration['grid_fields']['grid_title'];
    $build['#display_title'] = $this->configuration['grid_fields']['grid_display_title'];
    $build['#position_media'] = $this->configuration['grid_fields']['grid_position_media'];
    $build['#text']['#markup'] = $this->configuration['grid_fields']['grid_text']['value'];
    $build['#link'] = $this->configuration['grid_fields']['grid_link'];
    $build['#classes'] = $this->configuration['grid_fields']['grid_classes'];

    return $build;
  }

}
