<?php

namespace Drupal\layout_builder_kit\Plugin\Block\LBKImage;

use Drupal\Core\Config\ConfigManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\file\Entity\File;
use Drupal\layout_builder_kit\Plugin\Block\LBKBaseComponent;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a 'LBKImage' block.
 *
 * @Block(
 *  id = "lbk_image",
 *  admin_label = @Translation("Image (LBK)"),
 * )
 */
class LBKImage extends LBKBaseComponent implements ContainerFactoryPluginInterface {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\Core\Config\ConfigManagerInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigManagerInterface
   */
  protected $configManager;

  /**
   * Constructs a new GridComponent object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param string $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The EntityTypeManagerInterface service.
   * @param \Drupal\Core\Config\ConfigManagerInterface $config_manager
   *   The ConfigManagerInterface service.
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityTypeManagerInterface $entity_type_manager,
    ConfigManagerInterface $config_manager
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityTypeManager = $entity_type_manager;
    $this->configManager = $config_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('config.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'image_component_fields' => [
        'title' => 'Image (LBK)',
        'display_title' => TRUE,
        'image' => [],
        'image_style' => 'none',
        'position' => 'left',
        'text' => [],
        'overlay_text' => 'none',
        'title_position' => 'title_on_top',
        'classes' => '',
      ],
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   *
   * Creates a generic configuration form for all block types. Individual
   * block plugins can add elements to this form by overriding
   * BlockBase::blockForm(). Most block plugins should not override this
   * method unless they need to alter the generic form elements.
   *
   * @see \Drupal\Core\Block\BlockBase::blockForm()
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $definition = $this->getPluginDefinition();
    $form['provider'] = [
      '#type' => 'value',
      '#value' => $definition['provider'],
    ];

    /*
    $form['admin_label'] = [
      '#type' => 'item',
      '#title' => $this->t('Block description'),
      '#plain_text' => $definition['admin_label'],
    ];
    */

    // Add context mapping UI form elements.
    $contexts = $form_state->getTemporaryValue('gathered_contexts') ?: [];
    $form['context_mapping'] = $this->addContextAssignmentElement($this, $contexts);
    // Add plugin-specific settings for this block type.
    $form += $this->blockForm($form, $form_state);
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    // Get value from ImageComponentSettingsForm's configuration.
    $configFactory = $this->configManager->getConfigFactory();
    $locationFolderConfig = $configFactory->get('layout_builder_kit.image_component');
    $locationFolder = $locationFolderConfig->get('image_location');

    $form['title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Title'),
      '#maxlength' => 255,
      '#default_value' => $this->configuration['image_component_fields']['title'],
      '#required' => TRUE,
      '#weight' => '1',
    ];

    $form['display_title'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display title'),
      '#default_value' => $this->configuration['image_component_fields']['display_title'],
      '#weight' => '2',
    ];

    if ($this->configuration['image_component_fields']['display_title']) {
      $form['title_position'] = [
          '#type' => 'select',
          '#title' => $this->t('Title position'),
          '#options' => [
              'title_on_top' => $this->t('Top'),
              'title_on_bottom' => $this->t('Bottom'),
          ],
          '#default_value' => $this->configuration['image_component_fields']['title_position'],
          '#weight' => '3',
      ];
    }

    $form['image'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Image'),
      '#upload_location' => ($locationFolder != NULL) ? 'public://' . $locationFolder : 'public://layout_builder_kit',
      '#multiple' => FALSE,
      '#default_value' => $this->configuration['image_component_fields']['image'],
      '#required' => TRUE,
      '#upload_validators' => [
        'file_validate_extensions' => ['gif png jpg jpeg'],
      ],
      '#weight' => '4',
    ];

    $form['image_style'] = [
      '#type' => 'select',
      '#title' => $this->t('Image style'),
      '#options' => [
        'none' => $this->t('None'),
        'thumbnail' => $this->t('Thumbnail'),
        'medium' => $this->t('Medium'),
        'large' => $this->t('Large'),
      ],
      '#default_value' => $this->configuration['image_component_fields']['image_style'],
      '#weight' => '5',
    ];

    $form['position'] = [
      '#type' => 'select',
      '#title' => $this->t('Position'),
      '#options' => [
        'left' => $this->t('Left'),
        'center' => $this->t('Center'),
        'right' => $this->t('Right'),
      ],
      '#default_value' => $this->configuration['image_component_fields']['position'],
      '#weight' => '6',
    ];

    $form['text'] = [
      '#type' => 'text_format',
      '#title' => $this->t('Overlay text'),
      '#default_value' => $this->configuration['image_component_fields']['text']['value'],
      '#format' => $this->configuration['image_component_fields']['text']['format'],
      '#weight' => '7',
    ];

    $form['overlay_text'] = [
      '#type' => 'select',
      '#title' => $this->t('Overlay text position'),
      '#options' => [
        'none' => $this->t('None'),
        'bottom' => $this->t('Bottom'),
      ],
      '#default_value' => $this->configuration['image_component_fields']['overlay_text'],
      '#weight' => '8',
    ];

    $form['classes'] = [
      '#type' => 'textfield',
      '#title' => $this->t('CSS class'),
      '#description' => $this->t('A list of CSS classes added to this component, separated by spaces.'),
      '#maxlength' => 64,
      '#default_value' => $this->configuration['image_component_fields']['classes'],
      '#weight' => '9',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['image_component_fields']['title'] = $form_state->getValue('title');
    $this->configuration['image_component_fields']['image'] = $form_state->getValue('image');
    $this->configuration['image_component_fields']['position'] = $form_state->getValue('position');
    $this->configuration['image_component_fields']['text'] = $form_state->getValue('text');
    $this->configuration['image_component_fields']['overlay_text'] = $form_state->getValue('overlay_text');
    $this->configuration['image_component_fields']['classes'] = $form_state->getValue('classes');
    $this->configuration['image_component_fields']['image_style'] = $form_state->getValue('image_style');

    if (!$form_state->getValue('display_title')) {
      $this->configuration['image_component_fields']['display_title'] = $form_state->getValue('display_title');
      $this->configuration['image_component_fields']['title_position'] = 'title_on_top';
    }
    else {
      $this->configuration['image_component_fields']['display_title'] = $form_state->getValue('display_title');
      $this->configuration['image_component_fields']['title_position'] = $form_state->getValue('title_position');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $build = [];
    // Add template.
    $build['#theme'] = 'LBKImage';
    // Attach library.
    $build['#attached']['library'] = ['layout_builder_kit/image-styling'];

    if (!empty($this->configuration['image_component_fields']['image'])) {
      $imageFileId = implode($this->configuration['image_component_fields']['image']);
      $image = File::load($imageFileId);
      if ($image != NULL) {
        $image->setPermanent();
        $image->save();

        if ($this->configuration['image_component_fields']['image_style'] == "none") {
          $build['#image'] = [
            '#theme' => 'image',
            '#uri' => $image->getFileUri(),
          ];
        }
        else {
          $build['#image'] = [
            '#theme' => 'image_style',
            '#style_name' => $this->configuration['image_component_fields']['image_style'],
            '#uri' => $image->getFileUri(),
          ];
        }
      }
    }

    $build['#title'] = $this->configuration['image_component_fields']['title'];
    $build['#display_title'] = $this->configuration['image_component_fields']['display_title'];
    $build['#position'] = $this->configuration['image_component_fields']['position'];
    $build['#text']['#markup'] = $this->configuration['image_component_fields']['text']['value'];
    $build['#overlay_text'] = $this->configuration['image_component_fields']['overlay_text'];
    $build['#title_position'] = $this->configuration['image_component_fields']['title_position'];
    $build['#classes'] = $this->configuration['image_component_fields']['classes'];

    return $build;
  }

}
