<?php

namespace Drupal\layout_builder_kit\Plugin\Block\LBKTab;

use Drupal\block_content\Entity\BlockContent;
use Drupal\Core\Form\FormStateInterface;
use Drupal\layout_builder_kit\Plugin\Block\LBKBaseComponent;
use Symfony\Component\DependencyInjection\ContainerInterface;

use Drupal\block\Entity\Block;
use Drupal\Core\Block\BlockManager;
use Drupal\Core\Extension\Extension;
use Drupal\Core\Plugin\Context\LazyContextRepository;

/**
 * Provides a 'LBKTab' block.
 *
 * @Block(
 *  id = "lbk_tab",
 *  admin_label = @Translation("Tab (LBK)"),
 * )
 */
class LBKTab extends LBKBaseComponent {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
        'lbk_tab' => [
          'tab_content' => [],
        ],
        'tabs_default_text' => $this->t('No tab content.'),
      ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $config = $this->getConfiguration();

    $form['#tree'] = TRUE;
    $form['items_fieldset'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Tabs'),
      '#prefix' => '<div id="items-fieldset-wrapper">',
      '#suffix' => '</div>'
    ];

    if (!$form_state->has('items')) {
      $form_state->set('items', $config['lbk_tab']['tab_content']);
    }

    $tabs = $form_state->get('items');
    $numTabs = count($tabs);
    for ($i = 0; $i < $numTabs; $i++) {

      if ($tabs[$i]['type'] === 'custom_text') {
        // Tab label.
        $form['items_fieldset']['items'][$i]['name_tab'] = [
          '#type' => 'textfield',
          '#title' => $this->t('Tab label'),
          '#default_value' => $tabs[$i]['name_tab'],
          '#maxlength' => 64,
          '#size' => 64,
          '#weight' => '0',
        ];

        // Content Tab.
        $form['items_fieldset']['items'][$i]['text_format'] = [
          '#type' => 'text_format',
          '#title' => $this->t('Rich text'),
          '#default_value' => $tabs[$i]['text_format']['value'],
          '#format' => $tabs[$i]['text_format']['format'],
          '#weight' => '1',
        ];

      } else {
        if ($tabs[$i]['type'] === 'block') {
          // Tab label.
          $form['items_fieldset']['items'][$i]['name_tab'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Tab label'),
            '#default_value' => $tabs[$i]['name_tab'],
            '#maxlength' => 64,
            '#size' => 64,
            '#weight' => '0',
          ];

          // Block Tab.
          $form['items_fieldset']['items'][$i]['name'] = [
            '#type' => 'select',
            '#title' => $this->t('Content Block'),
            '#options' => $this->getBlocks(),
            '#default_value' => $tabs[$i]['name'],
            '#weight' => '1',
          ];

        }
      }
    }

    $form['items_fieldset']['actions'] = [
      '#type' => 'actions'
    ];

    $form['items_fieldset']['actions']['add_custom_text'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add custom text'),
      '#submit' => [[$this, 'addCustomText']],
      '#ajax' => [
        'callback' => [$this, 'addCustomTextCallback'],
        'wrapper' => 'items-fieldset-wrapper'
      ],
    ];

    $form['items_fieldset']['actions']['add_block_tab'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add block'),
      '#submit' => [[$this, 'addBlock']],
      '#ajax' => [
        'callback' => [$this, 'addBlockCallback'],
        'wrapper' => 'items-fieldset-wrapper'
      ],
    ];

    if ($numTabs >= 1) {
      $form['items_fieldset']['actions']['remove_item'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove'),
        '#submit' => [[$this, 'removeLastBlock']],
        '#ajax' => [
          'callback' => [$this, 'removeLastBlockCallback'],
          'wrapper' => 'items-fieldset-wrapper'
        ],
      ];
    }

    $form['tabs_default_text'] = [
      '#type' => 'textfield',
      '#title' => $this->t('No tabs text.'),
      '#default_value' => $this->configuration['tabs_default_text'],
      '#maxlength' => 64,
      '#size' => 64,
      '#weight' => '1',
    ];

    return $form;
  }

  /**
   * @param array $form
   * @param FormStateInterface $form_state
   */
  public function addCustomText(array &$form, FormStateInterface $form_state) {
    $items = [];
    if ($form_state->has('items')) {
      $items = $form_state->get('items');
      $nextItem = count($items);
      $items[$nextItem]['type'] = 'custom_text';
    } else {
      $nextItem = count($items);
      $items[$nextItem]['type'] = 'custom_text';
    }

    $form_state->set('items', $items);
    $form_state->setRebuild();
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return mixed
   */
  public function addCustomTextCallback(array &$form, FormStateInterface $form_state) {
    return $form['settings']['items_fieldset'];
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function addBlock(array &$form, FormStateInterface $form_state) {
    $items = [];
    if ($form_state->has('items')) {
      $items = $form_state->get('items');
      $nextItem = count($items);
      $items[$nextItem]['type'] = 'block';
    } else {
      $nextItem = count($items);
      $items[$nextItem]['type'] = 'block';
    }

    $form_state->set('items', $items);
    $form_state->setRebuild();
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return mixed
   */
  public function addBlockCallback(array &$form, FormStateInterface $form_state) {
    return $form['settings']['items_fieldset'];
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   */
  public function removeLastBlock(array &$form, FormStateInterface $form_state) {
    $items = [];
    if ($form_state->has('items')) {
      $items = $form_state->get('items');
      array_pop($items);
    }

    $form_state->set('items', $items);
    $form_state->setRebuild();
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return mixed
   */
  public function removeLastBlockCallback(array &$form, FormStateInterface $form_state) {
    return $form['settings']['items_fieldset'];
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    $this->configuration['tabs_default_text'] = $form_state->getValue('tabs_default_text');

    // Set to empty before save all again.
    $this->configuration['lbk_tab']['tab_content'] = [];

    foreach ($form_state->getValues() as $key => $value) {
      if ($key === 'items_fieldset') {
        if (isset($value['items'])) {
          $items = $value['items'];
          $tabs = $form_state->get('items');
          for ($i = 0; $i < count($tabs); $i++) {
            if ($tabs[$i]['type'] === 'custom_text') {
              $this->configuration['lbk_tab']['tab_content'][$i]['type'] = 'custom_text';
              $this->configuration['lbk_tab']['tab_content'][$i]['name_tab'] = $items[$i]['name_tab'];
              $this->configuration['lbk_tab']['tab_content'][$i]['text_format']['value'] = $items[$i]['text_format']['value'];
              $this->configuration['lbk_tab']['tab_content'][$i]['text_format']['format'] = $items[$i]['text_format']['format'];
            } else {
              if ($tabs[$i]['type'] === 'block') {
                $this->configuration['lbk_tab']['tab_content'][$i]['type'] = 'block';
                $this->configuration['lbk_tab']['tab_content'][$i]['name_tab'] = $items[$i]['name_tab'];
                $this->configuration['lbk_tab']['tab_content'][$i]['name'] = $items[$i]['name'];
              }
            }

          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $tabs = [];
    //TODO: Create an array to build tabs in template.
    if (isset($this->configuration['lbk_tab']['tab_content'])) {
      for ($i = 0; $i < count($this->configuration['lbk_tab']['tab_content']); $i++) {
        if ($this->configuration['lbk_tab']['tab_content'][$i]['type'] === 'block') {
          $tabs[$i]['type'] = $this->configuration['lbk_tab']['tab_content'][$i]['type'];
          $tabs[$i]['name_tab'] = $this->configuration['lbk_tab']['tab_content'][$i]['name_tab'];

          // Render Block.
          if ((int) $this->configuration['lbk_tab']['tab_content'][$i]['name'] > 0) {
            // Render Content Block.
            $bid = $this->configuration['lbk_tab']['tab_content'][$i]['name'];
            $block = BlockContent::load($bid);
            if ($block instanceof BlockContent) {
              $renderedBlock = \Drupal::entityTypeManager()->getViewBuilder('block_content')->view($block);
            }
            else {
              if ($this->configuration['lbk_tab']['tab_content'][$i]['name'] == 'none') {
                $renderedBlock = $this->t("There is no any content block yet.");
              }
              else {
                $renderedBlock = $this->t("Sorry this Block was deleted.");
              }
            }

          }
          else {
            // Render any content block.
            switch ($this->configuration['lbk_tab']['tab_content'][$i]['name']) {
              case "custom":
                $renderedBlock = $this->t("Sorry this option is incorrect.");
                break;

              case "contrib":
                $renderedBlock = $this->t("Sorry this option is incorrect.");
                break;

              case "core" :
                $renderedBlock = $this->t("Sorry this option is incorrect.");
                break;

              default:
                $blockEntity = Block::load($this->configuration['lbk_tab']['tab_content'][$i]['name']);
                $renderedBlock = \Drupal::entityTypeManager()
                  ->getViewBuilder('block')
                  ->view($blockEntity);

            }
          }
          $tabs[$i]['name'] = $renderedBlock;
        }
        else {
          $tabs[$i] = $this->configuration['lbk_tab']['tab_content'][$i];
        }
      }
    }

    $build = [];
    $build['#theme'] = 'LBKTab';
    $build['#attached']['library'] = ['layout_builder_kit/tab-styling'];
    $build['#tabs_default_text']['#markup'] = $this->configuration['tabs_default_text'];
    $build['#tabs'] = $tabs;
    return $build;
  }

  /**
   * Get all Content Blocks.
   *
   * @return array
   */
  private function getContentBlocks() {
    $blocks = BlockContent::loadMultiple();

    $options = ['custom' => $this->t('----Custom----')];

    foreach ($blocks as $key => $value) {
      if ($value instanceof BlockContent)
      $options[$value->id()] = $value->label();
    }

    return $options;
  }

  /**
   * Get all Content Plugin Blocks.
   *
   * @return array
   */
  private function getContentPluginBlocks() {
    // Get in Drop-down all Content Plugin Blocks.
    $blocksList = ['contrib' => $this->t('----Modules----')];

    // Load all Plugin Block entities.
    $blocks = Block::loadMultiple();

    $moduleHandler = \Drupal::moduleHandler();
    $modules = $moduleHandler->getModuleList();

    foreach ($modules as $key => $value) {
      if ($value instanceof Extension) {
        if ($value->getType() === 'module') {
          $pathBlock =  explode('/', $value->getPathname());
          if (in_array('contrib', $pathBlock) || in_array('custom', $pathBlock)) {
            foreach ($blocks as $keyBlock => $valueBlock) {
              if ($valueBlock instanceof Block) {
                $dependenciesModuleBlock = $valueBlock->getDependencies()['module'];
                if (in_array($value->getName(), $dependenciesModuleBlock)) {
                  $blocksList[$valueBlock->id()] = $valueBlock->label();
                }
              }
            }
          }
        }
      }
    }

    return $blocksList;
  }

  /**
   * Get all System Plugin Blocks.
   *
   * @return array
   */
  private function getSystemPluginBlocks() {
    // Get in Drop-down all Content Plugin Blocks.
    $blocksList = ['core' => $this->t('----System----')];
    // Load all Plugin Block entities.
    $blocks = Block::loadMultiple();

    $moduleHandler = \Drupal::moduleHandler();
    $modules = $moduleHandler->getModuleList();

    foreach ($modules as $key => $value) {
      if ($value instanceof Extension) {
        if ($value->getType() === 'module') {
          $pathModule =  explode('/', $value->getPathname());
          if (in_array('core', $pathModule)) {
            foreach ($blocks as $keyBlock => $valueBlock) {
              if ($valueBlock instanceof Block) {
                $dependenciesModuleBlock = $valueBlock->getDependencies()['module'];
                if (in_array($value->getName(), $dependenciesModuleBlock)) {
                  $blocksList[$valueBlock->id()] = $valueBlock->label();
                }
              }
            }
          }
        }
      }
    }
    return $blocksList;
  }

  /**
   * Get All Blocks together.
   *
   * @return array
   */
  private function getBlocks(){
    $contentAndContentPluginBlocks = array_merge($this->getContentBlocks(), $this->getContentPluginBlocks());
    $allBlocks = array_merge($contentAndContentPluginBlocks, $this->getSystemPluginBlocks());

    return $allBlocks;
  }

}
